'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var Koa = require('koa');
var KoaRouter = require('koa-router');
var cheerio = require('cheerio');
var fs = require('fs');
var url = require('url');
var server = require('react-dom/server');
var path = require('path');
var history$1 = require('history');
var path2Regexp = _interopDefault(require('path-to-regexp'));
var send = _interopDefault(require('koa-send'));

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
}

function __awaiter(thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

const DEFAULT_WEBPACK_SERVER_PORT = 8080;
const DEFAULT_DEV_SERVER_PORT = 3030;
const DEFAULT_CLIENT_PUBLIC_PATH = '/static/';
const DEFAULT_SERVER_PUBLIC_PATH = '/';
const rootDirectory = process.cwd();
function getConfig() {
    const htmlFilename = 'client.html';
    const staticDirName = 'static';
    const buildDirName = 'build';
    return {
        htmlFilename,
        buildDirName,
        staticDirName,
        clientChunkName: 'app.js',
        serverChunkName: 'server.js',
        webpackServerPort: DEFAULT_WEBPACK_SERVER_PORT,
        serverPort: DEFAULT_DEV_SERVER_PORT,
        clientPublicPath: DEFAULT_CLIENT_PUBLIC_PATH,
        serverPublicPath: DEFAULT_SERVER_PUBLIC_PATH,
        htmlTemplatePath: path.join(rootDirectory, 'src/index.html'),
        htmlPath: path.join(rootDirectory, staticDirName, 'client.html'),
        buildDirectory: path.join(rootDirectory, buildDirName),
        staticDirectory: path.join(rootDirectory, staticDirName),
        srcDirectory: path.join(rootDirectory, 'src'),
        customConfigFile: path.join(rootDirectory, 'server-renderer.config.js'),
        envFiles: [
            '.env',
            `.env.${process.env.NODE_ENV}`
        ],
    };
}

const routerStore = {
    location: {
        pathname: '',
        search: '',
        state: null,
        hash: '',
    },
};
const RouterContext = React.createContext(routerStore);

const history = 'undefined' === typeof window
    ? history$1.createMemoryHistory()
    : history$1.createBrowserHistory();
class Router extends React.Component {
    constructor(props) {
        super(props);
        this.fetchInitialProps = (comp, pathname) => __awaiter(this, void 0, void 0, function* () {
            if (comp.getInitialProps) {
                this.setState({ loading: true });
                try {
                    const initialProps = yield comp.getInitialProps(pathname);
                    this.setState(({ component }) => {
                        if (comp !== component) {
                            return null;
                        }
                        return {
                            data: initialProps,
                            loading: false,
                        };
                    });
                }
                catch (error) {
                    this.setState(({ component }) => {
                        if (comp !== component) {
                            return null;
                        }
                        return { loading: false, error };
                    });
                }
            }
        });
        const { routes, location, pageProps } = props;
        const matchedRoute = this.getMatchedRoute(routes, location);
        this.state = {
            routerStore: {
                location: {
                    pathname: location,
                    search: '',
                    state: null,
                    hash: '',
                },
            },
            loading: false,
            data: pageProps,
            error: null,
            component: matchedRoute
                ? matchedRoute.component
                : null
        };
    }
    componentDidMount() {
        const { routes } = this.props;
        this.unlisten = history.listen((location) => {
            const matched = this.getMatchedRoute(routes, location.pathname);
            const component = matched ? matched.component : null;
            this.setState(({ routerStore }) => {
                return {
                    routerStore: Object.assign({}, routerStore, { location }),
                    component,
                    data: null,
                    error: null,
                    loading: false,
                };
            }, () => {
                this.fetchInitialProps(component, location.pathname);
            });
        });
    }
    render() {
        const { AppContainer } = this.props;
        const { routerStore, component, data, loading, error } = this.state;
        return (React.createElement(RouterContext.Provider, { value: routerStore },
            React.createElement(AppContainer, null, this.renderContent(component, data, loading, error))));
    }
    componentWillUnmount() {
        this.unlisten();
    }
    renderContent(Component, data, loading, error) {
        if (Component) {
            return (React.createElement(Component, { data: data, loading: loading, error: error }));
        }
        return null;
    }
    getMatchedRoute(routes, pathname) {
        return routes.find(route => {
            return path2Regexp(pathname).test(route.path);
        });
    }
}
Router.push = history.push;
Router.replace = history.replace;
Router.go = history.go;
Router.goBack = history.goBack;

const Link = (_a) => {
    var { to, className = '', activeClassName = '', onClick } = _a, restProps = __rest(_a, ["to", "className", "activeClassName", "onClick"]);
    const { location } = React.useContext(RouterContext);
    const matched = path2Regexp(location.pathname).test(to);
    const cls = className + matched ? activeClassName : '';
    const handleClick = (evt) => {
        if (onClick) {
            return onClick(evt);
        }
        if (!evt.isDefaultPrevented()) {
            evt.preventDefault();
            Router.push(to);
        }
    };
    return (React.createElement("a", Object.assign({ href: to, className: cls, onClick: handleClick }, restProps)));
};

const config = getConfig();
const isDev = process.env.NODE_ENV === 'development';
class Server {
    constructor(opts) {
        this.port = config.serverPort;
        this.clientChunkPath = new url.URL(config.clientChunkName, `http://localhost:${config.webpackServerPort}${config.clientPublicPath}`);
        this.container = opts.container;
        this.AppContainer = opts.AppContainer || React.Fragment;
        this.routes = opts.routes;
        const htmlPath = isDev ? config.htmlTemplatePath : config.htmlPath;
        this.originalHTML = fs.readFileSync(htmlPath, 'utf-8');
    }
    start() {
        const app = new Koa();
        const router = new KoaRouter();
        if (!isDev) {
            app.use(this.serveFiles);
        }
        router.get('*', this.handleRequest.bind(this));
        app.use(router.routes());
        app.listen(this.port, () => {
            console.log('Server listen on: http://localhost:' + this.port);
        });
    }
    handleRequest(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            const routes = this.routes;
            const pathname = ctx.url;
            const matchedRoute = routes.find(route => {
                return path2Regexp(pathname).test(route.path);
            });
            if (matchedRoute) {
                const AppContainer = this.AppContainer;
                const pageProps = yield this.getInitialProps(pathname, matchedRoute);
                const content = server.renderToString(React.createElement(Router, { location: pathname, AppContainer: AppContainer, pageProps: pageProps, routes: routes }));
                ctx.body = this.renderHTML(content, pageProps);
            }
            else {
                ctx.body = 'Page not found';
            }
        });
    }
    serveFiles(ctx, next) {
        return __awaiter(this, void 0, void 0, function* () {
            const staticDirName = config.staticDirName;
            const staticDirectory = config.staticDirectory;
            const path = ctx.path;
            if (path.startsWith(`/${staticDirName}/`)) {
                yield send(ctx, path.replace(`/${staticDirName}/`, ''), { root: staticDirectory });
            }
            else {
                yield next();
            }
        });
    }
    renderHTML(content, pageProps) {
        const $ = cheerio.load(this.originalHTML);
        $(this.container).html(content);
        $('head').append(`
      <script type='text/javascript'>
          __APP_DATA__='${encodeURIComponent(JSON.stringify({ pageProps }))}'
      </script>
    `);
        if (isDev) {
            $('body').append(`
        <script type='text/javascript' src='${this.clientChunkPath}'></script>
      `);
        }
        return $.html();
    }
    getInitialProps(pathname, matchedRoute) {
        return __awaiter(this, void 0, void 0, function* () {
            const { component } = matchedRoute;
            if (component.getInitialProps) {
                return yield component.getInitialProps(pathname);
            }
            return {};
        });
    }
}
function render(opts) {
    const server = new Server(opts);
    server.start();
}

Object.keys(history$1).forEach(function (key) { exports[key] = history$1[key]; });
exports.render = render;
exports.Router = Router;
exports.Link = Link;
